#!/usr/bin/env nix-shell
#!nix-shell --pure -i runghc -p "haskellPackages.ghcWithPackages (pkgs: with pkgs; [ json ])"

-- https://wiki.haskell.org/Getting_the_current_date
-- https://hackage.haskell.org/package/time-1.13/docs/Data-Time.html datetime modules overview
-- https://hackage.haskell.org/package/time-1.12.2/docs/Data-Time-Clock.html
-- https://hackage.haskell.org/package/time-1.12.2/docs/Data-Time-Calendar.html
-- https://hackage.haskell.org/package/time-1.12.2/docs/Data-Time-LocalTime.html
-- https://hackage.haskell.org/package/cgi-3001.5.0.1/docs/Network-CGI.html
-- https://mmhaskell.com/monads/reader-writer

-- https://hackage.haskell.org/package/type-of-html-1.6.2.0/docs/Html.html
-- https://github.com/knupfer/type-of-html

-- Overview of all language extension
-- https://downloads.haskell.org/ghc/latest/docs/users_guide/exts/table.html
-- {-# LANGUAGE DuplicateRecordFields #-}

-- super sick extension learned here https://stackoverflow.com/questions/38052553/haskell-record-pattern-matching
{-# LANGUAGE RecordWildCards #-}

import Data.Time
import Data.List
--import Control.Monad.Reader
import System.Environment
import Network.CGI
import Caster
import Webpage
--import Control.Applicative
import Data.Maybe

casters :: CGI [Caster]
casters = do
  makeCaster <- makeCaster'
  return [ makeCaster "山岸愛梨" $ YearMonthDay 1987 06 09
         , makeCaster "松雪彩花" $ YearMonthDay 1991 09 06
         , makeCaster "武藤彩芽" $ YearMonthDay 1996 08 31
         , makeCaster "眞家泉" $ YearMonthDay 1990 09 07
         , makeCaster "魚住茉由" $ YearMonthDay 1998 12 06
         , makeCaster "戸北美月" $ YearMonthDay 1999 12 29
         , makeCaster "青原桃香" $ YearMonthDay 1999 01 05
         , makeCaster "高山奈々" $ YearMonthDay 1994 06 17
         , makeCaster "高安奈緒子" $ YearMonthDay 1992 11 22
         , makeCaster "小林李衣奈" $ YearMonthDay 1999 11 17
         , makeCaster "大島璃音" $ YearMonthDay 1999 03 19
         , makeCaster "檜山沙耶" $ YearMonthDay 1993 10 27
         , makeCaster "江川清音" $ YearMonthDay 1989 12 03
         , makeCaster "小川千奈" $ YearMonthDay 1999 06 30
         , makeCaster "駒木結衣" $ YearMonthDay 1996 06 09
         , makeCaster "岡本結子リサ" $ YearMonthDay 2000 01 30
         , makeCaster "白井ゆかり" $ YearMonthDay 1991 06 25
         , makeCaster "内田侑希" $ YearMonthDay 1995 12 08 ]

jptz :: TimeZone
jptz = TimeZone (9 * 60) False "JST" -- define JST time zone as 9 hours past UTC https://time.is/ja/Japan

jplocaltime :: CGI LocalTime
jplocaltime = (utcToLocalTime jptz) <$> liftIO getCurrentTime

jpdate :: CGI Day
jpdate = (readInput "date" :: CGI (Maybe Day)) >>=
         (\x -> do
             jplocalday' <- fmap localDay jplocaltime
             return $ fromMaybe jplocalday' x)

jpyear :: CGI Year
jpyear = fmap ((\(y,_,_) -> y) . toGregorian) jpdate

-- not used
dayToTime :: Day -> LocalTime
dayToTime day = LocalTime day midnight

nextBday :: CGI (Day -> Day)
nextBday = do
  today <- jpdate
  thisyear <- jpyear
  return (\dob ->
             let (_,dobD,dobM) = toGregorian dob
                 bdaythisyear = fromGregorian thisyear dobD dobM
                 nextBdayYear = if bdaythisyear < today then thisyear + 1 else thisyear
             in fromGregorian nextBdayYear dobD dobM)
 
findAge :: CGI (Day -> Age)
findAge = do
    today <- jpdate
    nextbday <- nextBday
    return (\dob ->
              let monthsAndDays = diffGregorianDurationClip today dob
                  totalMonths = cdMonths monthsAndDays
                  y = quot totalMonths 12
                  m = rem totalMonths 12
                  d = cdDays monthsAndDays
                  countdown = diffDays (nextbday dob) today
              in Age y m d countdown)

makeCaster' :: CGI (String -> Day -> Caster)
makeCaster' = do
  findAge' <- findAge
  return (\name bday -> Caster name bday $ findAge' bday)

sortByDOB :: [Caster] -> [Caster]
sortByDOB = sortOn casterBday

sortByCd :: [Caster] -> [Caster]
sortByCd = sortOn (ageCountdown.casterAge)

-- jsonoutput :: CGI CGIResult
-- jsonoutput = do
--   outputFPS $ encode $ toJSONList datalist
--  liftIO putStrLn $ "日本での現在時刻 " ++ (show jptime)
  
htmloutput :: CGI CGIResult
htmloutput =  do
  jpdate' <- jpdate
  sortcrit <- getInput "sorton"
  casters' <- casters
  let sortmethod = if sortcrit == Just "cd" then sortByCd else sortByDOB
  setHeader "Content-type" "text/html; charset=utf-8" :: CGI ()
  outputFPS $ webpage jpdate' $ sortmethod casters' :: CGI CGIResult

main :: IO ()
main = do
  console <- lookupEnv "TERM"
  if isNothing console then runCGI htmloutput else print "hello"
